#------------------------------------------------------------------
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
# $Header: /project/isv-agent/VProAgents/SybaseRMA/SybaseRMA.pm,v 1.1 2013/09/17 07:25:02 lpatil Exp $
#
# $Author: lpatil $
# $Date: 2013/09/17 07:25:02 $
# $Revision: 1.1 $
#
# Provides: SybaseRMA
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;
use warnings;
#----------------------
# Declare the package..
#----------------------
package SybaseRMA;
use ag_i18n_inc;

my ( $CLUS_HOME, $sPrefix ) = ();

#---------------------
# Setup VCS home..
#---------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.0
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcs";
}
else { # VCS
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /(...)$/;

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = "$CLUS_HOME/bin/hares";
my $sCmdHatype = "$CLUS_HOME/bin/hatype";
my $sCmdVcsDecrypt = "$CLUS_HOME/sbin/${sPrefix}decrypt";

=pod

 -------------------------------------------------------------------------
 Function :  Integer IsSybasePresent ( IN String  EntryPointName,
                              IN String  Homedir,
                              IN String  ServerProc,
                              IN String  PS,
                              IN Boolean InSolarisZone,
                              IN String  SolarisZoneString);

 Description : Determine if the Sybase process is running

 Parameters :

 $localHomedir             		Sybase Home folder
 $localServerProc            	Sybase Server
 $localPS		                Unix cmd for process status
 $localInSolarisZone            Running in Solaris Zone
 $localSolarisZoneString        Solaris Zone format string

 Returns : process id where
           0 not running
           >0 pid of running process
 -------------------------------------------------------------------------

=cut


sub IsSybasePresent 
{

	#----------------------------------------
	# Set up local vars from passed in ones..
	#----------------------------------------
	my ( $localOwner,
		$localHomedir,
		$localServerProc,
		$localInSolarisZone,
		$localSolarisZoneString ) = @_;
	$localSolarisZoneString = '' unless defined $localSolarisZoneString;      # Avoid undefs..	
	my $localPS = GetPS($localOwner);
	my @lsDet = ();

	#-----------------
	# Announce Entry..
	#-----------------
	my $sMyName = ( caller 0 )[3];

	VCSAG_LOGDBG_MSG(1, "Subroutine [$sMyName] called with: 
		HomeDir = [$localHomedir] 
		ServerProc = [$localServerProc]
		InSolarisZone = [$localInSolarisZone]
		localSolarisZoneString = [$localSolarisZoneString]
		", 0) ;


	my $iProcessID = 0;
	my $base2 = (split(/\//, $localHomedir))[-1];
	VCSAG_LOGDBG_MSG(1, "Extracting instance name as [$base2] from [$localHomedir].", 0);

	# 
	# Example of SybaseRMA process
	#
	# /remote/repeng1/sybase/C01_REP/shared/JRE-7_0_7_64BIT/bin/java -Xms128m -XX:MaxPermSize=128m -XX:+HeapDumpOnOutOfMemoryError -Dcom.sybase.home=/remote/repeng1/sybase/C01_REP -Dcom.sybase.ua.toplevel=/remote/repeng1/sybase/C01_REP/SCC-3_2 -Dcom.sybase.ua.home=/remote/repeng1/sybase/C01_REP/SCC-3_2 -Dcom.sybase.platform=linux -Djava.library.path=/remote/repeng1/sybase/C01_REP/SCC-3_2/rtlib:/remote/repeng1/sybase/C01_REP/SCC-3_2/bin/sa/bin_linux64 -Djava.security.policy=/remote/repeng1/sybase/C01_REP/SCC-3_2/conf/java.policy -Djava.util.logging.manager=com.sybase.ua.util.logging.bridge.JavaToLog4jLogManager -classpath /remote/repeng1/sybase/C01_REP/SCC-3_2/common/lib/commons-cli-1.1.jar:/remote/repeng1/sybase/C01_REP/SCC-3_2/server/lib/uaf-startup.jar com.sybase.ua.startup.Startup

	VCSAG_LOGDBG_MSG(6, "Search pattern [$localServerProc]", 0);
	VCSAG_LOGDBG_MSG(6, "Search pattern [$base2]", 0);

	open (localPS, "$localPS | ") || die "Unable to pipe from $localPS\n";
	while (<localPS>) {

		VCSAG_LOGDBG_MSG(6, "Process commandline [$_]", 0);
		#  Process next entry unless the current entry contains the $localServerProc string
		next unless /\b$localServerProc\b/;
		VCSAG_LOGDBG_MSG(5, "Found [$localServerProc] in [$_]", 0);
		#
		# The $localServerProc string was found in this entry, 
		# now check for the server name $Server and basedir or instance folder
		# if not found, go to the next entry, else process further
		#
		# examples: 	$localHomedir 	  = /../../D01_REP/..
		#      		$localServerProc   = repserver (or uaf-startup)
		# 
		next unless /\b$base2\b/;
		VCSAG_LOGDBG_MSG(5, "Found [$base2] in [$_]", 0);

		#
		# At this point we found both the $localServerProc and the instance directory strings
		#
		if ( 1 == $localInSolarisZone ) {
			# This entry point is running in the LocalZone, must be Solaris OS
			#VCSAG_LOGDBG_MSG(5, "Sybase server is possibly running in some local zone.", 0);
			#
			# Create an array of values using the format string for solaris
			# 
			@lsDet = ( $1, $2, $3, $4 ) if ( $_ =~ /$localSolarisZoneString/o );
			#VCSAG_LOGDBG_MSG(1, "Process details [ $_ ]", 0) ;
			if ( defined $lsDet[1]) {
				$iProcessID = IsProcInGlobalZone($lsDet[1]);
				next unless ( $iProcessID );
			} else {
				VCSAG_LOGDBG_MSG(5, "Could not get process id from [$_]", 0);
				next; 
			}
		} else {
			($iProcessID) = split;
			VCSAG_LOGDBG_MSG(5, "Got process id [$iProcessID] from [$_]", 0);
		}
		close(localPS);
		return $iProcessID;  # Found - sybase server/agent running
	}
	close(localPS);
	return $iProcessID; # Not Found ! sybase server/agent not running, returning 0
}

sub IsProcInGlobalZone {

	#
	# string representing the global zone and used to determine if entry point is running
	# in the global zone or note.
	my $globalZone = "global";

    my ( $PSP, $sPSPout ) = ();
	my $iProcPid = shift;
	$PSP = "/bin/ps -o pid,args,zone -p " . $iProcPid ;
	$sPSPout = `$PSP`;
	$sPSPout =~ s/\n//g;   # Chomp..
	VCSAG_LOGDBG_MSG(1, "Process id [$iProcPid] details [$sPSPout]", 0) ;
	if ( $sPSPout =~  /\b$globalZone\b/ ) {
		VCSAG_LOGDBG_MSG(1, "Found process id [$iProcPid] in global zone", 0) ;
		return $iProcPid;
	}
	return 0;
}

=pod

 -------------------------------------------------------------------------
 Function :  Integer GetPasswd ( IN String  EntryPointName,
                                 IN String  passwd_file );

 Description : Get the password from a file

 Parameters : 	$sEntryPointName     Entry point name
		$passwd_file                           File containing passwords

 Returns : null   - error processing password file
           string - password contained in file
 -------------------------------------------------------------------------

=cut

sub GetPasswd 
{
	#----------------------------------------
	# Set up local vars from passed in ones..
	#----------------------------------------
	my ( $sEntryPointName,$passwd_file ) = @_;
	
	
	#-----------------
	# Announce Entry..
	#-----------------
	my $sMyName = ( caller 0 )[3];

	VCSAG_LOGDBG_MSG(2, "Subroutine [$sMyName] called with: 
		EntryPointName [$sEntryPointName] and [$passwd_file]", 0) ;
	my $pass = "";
	# Open the password file
	if ( open ( PASSWDFILE, $passwd_file ) ) {
		# Get the password from the password file
		( $pass ) = grep { ( s/\n// || 1 ) } <PASSWDFILE>;
		# Make sure the password is not empty
		$pass = '' unless defined $pass;      # Avoid undefs..
		# close the password file
		close ( PASSWDFILE ) || VCSAG_LOG_MSG ( "E", "Could not close file [$passwd_file]", 28, $passwd_file );
	}
	else {
		VCSAG_LOG_MSG ( "E", "Could not open file [$passwd_file]", 29, $passwd_file );
	}
	# Return the password file value after attempting to read it from password file.
	return $pass;
}

=pod

 -------------------------------------------------------------------------
 Function :  execCmd ( IN String  Cmd
                       IN Boolean bRegister );

 Description : Execute given command and unregister based on flag

 Parameters :

 $cmd           Command to execute
 $bRegister     If bRegister = 1, registering for AMF notification
 		If bRegister = 0, un-registering for AMF notification

 Returns : 0 Success 
	   1 Failure
 -------------------------------------------------------------------------

=cut

sub execCmd {
	my ( $cmd, $cmdOut, $bRegister, $exitCode ) = ();
	$cmd = shift;
	$bRegister = shift;
	$cmdOut = `$cmd 2>&1`;
	$exitCode = $? >> 8;
	VCSAG_LOGDBG_MSG (3, "Executed command - [$cmd], command output - [$cmdOut], exitCode - [$exitCode]", 0);

	if (0 == $exitCode ) { 
		# SUCCESS
		return 0;
	}

	# Supress warning message for following exitCodes.
	# exitCode =  3 => ESRCH  => Already unregistered
	# exitCode = 17 => EEXIST => Already registered

	if ($bRegister ) {
		if (17 == $exitCode) {
			VCSAG_LOG_MSG ("W", "Resource is already registered with AMF.", 38);
			return 0;
		} else {
			VCSAG_LOG_MSG ("W", "amfregister for resource failed with error [$exitCode].", 30,$exitCode);
			return 1;
		}
	} else {
		if (3 == $exitCode) {
			VCSAG_LOG_MSG ("W", "Resource is already un-registered with AMF.", 37);
			return 0;
		} else {
			VCSAG_LOG_MSG ("W", "amfregister for resource failed with error [$exitCode].", 30,$exitCode);
			return 1;
		}
	} 
	return 1;
}

# Get platform specific PS command with required options

sub GetPS {
	my $Owner = shift;
	$ENV{'UNIX95'} = "";
	my $PS = "/bin/ps -ef -o pid,args";
	$PS = "/usr/ucb/ps axwwl" if ( $^O eq 'solaris' ); 
	$PS = "/bin/ps -U " . $Owner . " --cols=4096 -o pid,args" if ( $^O eq 'linux' );
	$PS = "/bin/ps -efx -o pid,args" if ( $^O eq 'hpux' );

	VCSAG_LOGDBG_MSG(1, "Using [$PS] command", 0) ;

	return $PS;
}

sub ProcessKill {
	my $iSignal = shift;
	my $my_repserver_pid = shift;
	if ($my_repserver_pid > 1) {
		#
		# Kill through SIGTERM is recommended by Sybase.(
		#
		if (kill ($iSignal, $my_repserver_pid) == 1) {
			VCSAG_LOGDBG_MSG(2, "killed process id $my_repserver_pid with signal $iSignal.", 0);
		} else {
			VCSAG_LOG_MSG("E", "killing process id $my_repserver_pid with signal $iSignal not successful.", 36, $my_repserver_pid, $iSignal);
		}
	}
}

1; # So PERL knows its included..
